"""Do the thing."""
import os
from typing import Dict, Any
from typing_extensions import Annotated
import yaml
import uvicorn
from playsound import playsound
from fastapi import FastAPI, Depends
from homespeaker.screenwaker import wake_the_screen

app = FastAPI()


def load_configuration() -> Dict[str, Any]:
    """Load the configuration file if it exists."""
    if os.environ.get("XDG_CONFIG_HOME"):
        config_path = os.path.join(
            os.environ.get("XDG_CONFIG_HOME"), "homespeaker", "config.yaml"
        )
    else:
        config_path = os.path.join(
            os.environ.get("HOME"), ".config", "homespeaker", "config.yaml"
        )
    with open(config_path, "r", encoding="utf8") as f:  # pylint: disable=invalid-name
        return yaml.safe_load(f)


@app.get("/playaudio/{key}")
def playaudio(key: str, config: Annotated[dict, Depends(load_configuration)]):
    """Play audio of some type."""
    playsound(config[key])
    return {}


@app.get("/wakescreen")
def wakescreen():
    """Wake the screen for the specified amount of time."""
    wake_the_screen()
    return {}

def homespeaker_entrypoint():
    """Launch the homespeaker server."""
    uvicorn.run(app, host="0.0.0.0", port=8000)



"""Wake the screen up code."""
import os
import subprocess


def wake_the_screen():
    """Wake the screen."""
    display_type = os.environ.get("XDG_SESSION_TYPE")
    if display_type == "wayland":
        subprocess.run([
            "dbus-send", "--session", "--dest=org.gnome.ScreenSaver",
            "--type=method_call", "/org/gnome/ScreenSaver",
            "org.gnome.ScreenSaver.SetActive", "boolean:false"
        ], shell=True, check=True)
    elif display_type == "x11":
        subprocess.run([
            "xset",
            "-display",
            os.environ.get("DISPLAY"),
            "s",
            "reset"
        ], shell=True, check=True)
    else:
        raise EnvironmentError(f"Unsupported display server {display_type}!")



"""Test main module."""
import pathlib
import pytest
from mockito import expect
from fastapi.testclient import TestClient
from homespeaker import main as sut

pytestmark = pytest.mark.usefixtures("unstub")  # pylint: disable=unused-argument


def test_load_configuration_with_home_ev(tmp_path: pathlib.Path):
    """Test the load_configuration function with a HOME environment variable."""
    config_dir = tmp_path / ".config" / "homespeaker/"
    config_dir.mkdir(parents=True)
    config_file_path = config_dir / "config.yaml"
    with open(
        config_file_path, "w", encoding="utf8"
    ) as f:  # pylint:disable=invalid-name
        f.write(
            (
                "---\n"
                "- cron:\n"
                "    schedule: 20 21 * * *\n"
                "    actions:\n"
                "      - light-screen:\n"
                "          duration: 8\n"
                "- cron:\n"
                "    schedule: 20 21 * * *\n"
                "    actions:\n"
                "      - play-sound:\n"
                "          src: alarm.mp3\n"
            )
        )
    expect(sut.os.environ).get("XDG_CONFIG_HOME").thenReturn(None)
    expect(sut.os.environ).get("HOME").thenReturn(tmp_path)
    result = sut.load_configuration()
    assert result == [
        {
            "cron": {
                "schedule": "20 21 * * *",
                "actions": [{"light-screen": {"duration": 8}}],
            }
        },
        {
            "cron": {
                "schedule": "20 21 * * *",
                "actions": [{"play-sound": {"src": "alarm.mp3"}}],
            }
        },
    ]


def test_load_configuration_with_xdg_home(tmp_path: pathlib.Path):
    """Test the load_configuration function with XDG_CONFIG_HOME environment variable."""
    config_dir = tmp_path / "homespeaker/"
    config_dir.mkdir(parents=True)
    config_file_path = config_dir / "config.yaml"
    with open(
        config_file_path, "w", encoding="utf8"
    ) as f:  # pylint:disable=invalid-name
        f.write(
            (
                "---\n"
                "- cron:\n"
                "    schedule: 20 21 * * *\n"
                "    actions:\n"
                "      - light-screen:\n"
                "          duration: 8\n"
                "- cron:\n"
                "    schedule: 20 21 * * *\n"
                "    actions:\n"
                "      - play-sound:\n"
                "          src: alarm.mp3\n"
            )
        )
    expect(sut.os.environ, times=2).get("XDG_CONFIG_HOME").thenReturn(tmp_path)
    result = sut.load_configuration()
    assert result == [
        {
            "cron": {
                "schedule": "20 21 * * *",
                "actions": [{"light-screen": {"duration": 8}}],
            }
        },
        {
            "cron": {
                "schedule": "20 21 * * *",
                "actions": [{"play-sound": {"src": "alarm.mp3"}}],
            }
        },
    ]


client = TestClient(sut.app)

def configuration():
    """Build a configuration."""
    return {"song1": "/some/path/some.where"}

@pytest.fixture
def override_config():
    """Override the configuration."""
    sut.app.dependency_overrides[sut.load_configuration] = configuration
    yield
    sut.app.dependency_overrides = {}


def test_playaudio(override_config):  # pylint: disable=redefined-outer-name, unused-argument
    """Test the playaudio function."""
    expect(sut).playsound("/some/path/some.where")
    response = client.get("/playaudio/song1")
    assert response.status_code == 200
    assert not response.json()

def test_wakescreen():
    """Test the wakescreen function."""
    expect(sut).wake_the_screen()
    response = client.get("/wakescreen")
    assert response.status_code == 200
    assert not response.json()

def test_homespeaker_entrypoint():
    """Test the homespeaker entrypoint."""
    expect(sut.uvicorn).run(sut.app, host="0.0.0.0", port=8000)
    sut.homespeaker_entrypoint()



"""Test the screenwaker module."""
import pytest
from mockito import expect
from homespeaker import screenwaker as sut

pytestmark = pytest.mark.usefixtures("unstub")  # pylint: disable=unused-argument

def test_wake_the_screen_x11():
    """Test the wake_the_screen function on x11."""
    with expect(sut.os.environ).get("XDG_SESSION_TYPE").thenReturn("x11"):
        with expect(sut.os.environ).get("DISPLAY").thenReturn(":0"):
            with expect(sut.subprocess).run(
                ["xset", "-display", ":0", "s", "reset"], shell=True, check=True
            ):
                sut.wake_the_screen()


def test_wake_screen_wayland():
    """Test the wake_the_screen function on wayland."""
    with expect(sut.os.environ).get("XDG_SESSION_TYPE").thenReturn("wayland"):
        with expect(sut.subprocess).run(
            [
                "dbus-send",
                "--session",
                "--dest=org.gnome.ScreenSaver",
                "--type=method_call",
                "/org/gnome/ScreenSaver",
                "org.gnome.ScreenSaver.SetActive",
                "boolean:false",
            ],
            shell=True,
            check=True,
        ):
            sut.wake_the_screen()

def test_wake_screen_other():
    """Test the wake_the_screen function on other."""
    with pytest.raises(EnvironmentError, match="Unsupported display server custom."):
        with expect(sut.os.environ).get("XDG_SESSION_TYPE").thenReturn("custom"):
            sut.wake_the_screen()
